//-----------------------------------------------------------------------------
// File: GetDXVer.cpp
//
// Desc: Demonstrates how applications can detect what version of DirectX
//       is installed.
//
// (C) Copyright 1995-1999 Microsoft Corp.  All rights reserved.
// Ported by Vadim Bodrov, TMT Development Corp.
//-----------------------------------------------------------------------------
unit GetDXVer;

interface

procedure GetDXVersion(var pdwDXVersion: DWORD; var pdwDXPlatform: DWORD);

implementation

uses Windows, DXLite;

//typedef HRESULT(WINAPI * DIRECTINPUTCREATE)( HINSTANCE, DWORD, LPDIRECTINPUT*,
//                                             IUnknown* );

type
    TDirectDrawCreate = ^function conv arg_stdcall (lpGUID: PGUID;
        var lplpDD: Pointer; pUnkOuter: Pointer): HResult;
    TDirectDrawCreateEx = ^function conv arg_stdcall (lpGUID: PGUID;
        var lplpDD: Pointer; iid: TREFIID; pUnkOuter: Pointer): HResult;
    TDirectInputCreate = ^function conv arg_stdcall (hinst: THandle;
        dwVersion: DWORD; var ppDI: PPDirectInput; punkOuter: PPUnknown): HResult;


//-----------------------------------------------------------------------------
// Name: GetDXVersion()
// Desc: This function returns two arguments:
//          dwDXVersion:
//            0x0000 = No DirectX installed
//            0x0100 = DirectX version 1 installed
//            0x0200 = DirectX 2 installed
//            0x0300 = DirectX 3 installed
//            0x0500 = At least DirectX 5 installed.
//            0x0600 = At least DirectX 6 installed.
//            0x0700 = At least DirectX 7 installed.
//          dwDXPlatform:
//            0                          = Unknown (This is a failure case)
//            VER_PLATFORM_WIN32_WINDOWS = Windows 9X platform
//            VER_PLATFORM_WIN32_NT      = Windows NT platform
//
//          Please note that this code is intended as a general guideline. Your
//          app will probably be able to simply query for functionality (via
//          QueryInterface) for one or two components.
//
//          Please also note:
//            "if (dxVer != 0x500) return FALSE;" is BAD.
//            "if (dxVer < 0x500) return FALSE;" is MUCH BETTER.
//          to ensure your app will run on future releases of DirectX.
//-----------------------------------------------------------------------------
procedure GetDXVersion(var pdwDXVersion: DWORD; var pdwDXPlatform: DWORD);
var
    hr:                  HResult;
    DDHinst:             THandle := 0;
    DIHinst:             THandle := 0;
    pDDraw:              PPDirectDraw := nil;
    pDDraw2:             PPDirectDraw2 := nil;
    pDD7:                PPDirectDraw7;
    PDirectDrawCreate:   TDirectDrawCreate := nil;
    PDirectDrawCreateEx: TDirectDrawCreateEx := nil;
    PDirectInputCreate:  TDirectInputCreate := nil;
    osVer:               TOSVersionInfo;
    pSurf:               PPDirectDrawSurface := nil;
    pSurf3:              PPDirectDrawSurface3 := nil;
    pSurf4:              PPDirectDrawSurface4 := nil;
    ddsd:                TDDSurfaceDesc;
begin
    // First get the windows platform
    osVer.dwOSVersionInfoSize := sizeof(osVer);
    if not GetVersionEx( osVer ) then
    begin
        pdwDXPlatform := 0;
        pdwDXVersion  := 0;
        exit;
    end;

    if osVer.dwPlatformId = VER_PLATFORM_WIN32_NT then
    begin
        pdwDXPlatform := VER_PLATFORM_WIN32_NT;

        // NT is easy... NT 4.0 is DX2, 4.0 SP3 is DX3, 5.0 is DX5
        // and no DX on earlier versions.
        if osVer.dwMajorVersion < 4 then
        begin
            pdwDXVersion := 0; // No DX on NT3.51 or earlier
            exit;
        end;

        if osVer.dwMajorVersion = 4 then
        begin
            // NT4 up to SP2 is DX2, and SP3 onwards is DX3, so we are at least DX2
            pdwDXVersion := $200;

            // We're not supposed to be able to tell which SP we're on, so check for dinput
            DIHinst := LoadLibrary( 'DINPUT.DLL' );
            if DIHinst = 0 then
            begin
                // No DInput... must be DX2 on NT 4 pre-SP3
                OutputDebugString( 'Couldn''t LoadLibrary DInput' + #13 );
                exit;
            end;

            PDirectInputCreate := GetProcAddress( DIHinst, 'DirectInputCreateA' );
            FreeLibrary( DIHinst );

            if PDirectInputCreate = nil then
            begin
                // No DInput... must be pre-SP3 DX2
                OutputDebugString( 'Couldn''t GetProcAddress DInputCreate' + #13 );
                exit;
            end;

            // It must be NT4, DX2
            pdwDXVersion := $300;  // DX3 on NT4 SP3 or higher
            exit;
        end;
        // Else it's NT5 or higher, and it's DX5a or higher: Drop through to
        // Win9x tests for a test of DDraw (DX6 or higher)
    end else
    begin
        // Not NT... must be Win9x
        pdwDXPlatform := VER_PLATFORM_WIN32_WINDOWS;
    end;

    // Now we know we are in Windows 9x (or maybe 3.1), so anything's possible.
    // First see if DDRAW.DLL even exists.
    DDHinst := LoadLibrary( 'DDRAW.DLL' );
    if DDHinst = 0 then
    begin
        pdwDXVersion := 0;
        FreeLibrary( DDHinst );
        exit;
    end;

    // See if we can create the DirectDraw object.
    PDirectDrawCreate := GetProcAddress( DDHinst, 'DirectDrawCreate' );
    if PDirectDrawCreate = nil then
    begin
        pdwDXVersion := 0;
        pdwDXPlatform := 0;
        FreeLibrary( DDHinst );
        OutputDebugString( 'Couldn''t LoadLibrary DDraw' + #13 );
        exit;
    end;

    hr := PDirectDrawCreate^( nil, pDDraw, nil );
    if FAILED(hr) then
    begin
        pdwDXVersion := 0;
        pdwDXPlatform := 0;
        FreeLibrary( DDHinst );
        OutputDebugString( 'Couldn''t create DDraw' + #13 );
        exit;
    end;

    // So DirectDraw exists.  We are at least DX1.
    pdwDXVersion := $100;

    // Let's see if IID_IDirectDraw2 exists.
    hr := pDDraw^^.QueryInterface^( pDDraw, IID_IDirectDraw2, pDDraw2 );
    if FAILED(hr) then
    begin
        // No IDirectDraw2 exists... must be DX1
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        OutputDebugString( 'Couldn''t QI DDraw2' + #13 );
        exit;
    end;

    // IDirectDraw2 exists. We must be at least DX2
    pDDraw2^^.Release^(pDDraw2);
    pdwDXVersion := $200;


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 3.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // DirectInput was added for DX3
    DIHinst := LoadLibrary( 'DINPUT.DLL' );
    if DIHinst = 0 then
    begin
        // No DInput... must not be DX3
        OutputDebugString( 'Couldn''t LoadLibrary DInput' + #13 );
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        exit;
    end;

    PDirectInputCreate := GetProcAddress( DIHinst, 'DirectInputCreateA' );

    if PDirectInputCreate = nil then
    begin
        // No DInput... must be DX2
        FreeLibrary( DIHinst );
        FreeLibrary( DDHinst );
        pDDraw^^.Release^(pDDraw);
        OutputDebugString( 'Couldn''t GetProcAddress DInputCreate' + #13 );
        exit;
    end;

    // DirectInputCreate exists. We are at least DX3
    pdwDXVersion := $300;
    FreeLibrary( DIHinst );

    // Can do checks for 3a vs 3b here


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 5.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // We can tell if DX5 is present by checking for the existence of
    // IDirectDrawSurface3. First, we need a surface to QI off of.
    ZeroMemory( @ddsd, sizeof(ddsd) );
    ddsd.dwSize         := sizeof(ddsd);
    ddsd.dwFlags        := DDSD_CAPS;
    ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;

    hr := pDDraw^^.SetCooperativeLevel^(pDDraw, 0, DDSCL_NORMAL );
    if FAILED(hr) then
    begin
        // Failure. This means DDraw isn't properly installed.
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        pdwDXVersion := 0;
        OutputDebugString( 'Couldn''t Set coop level' + #13 );
        exit;
    end;

    hr := pDDraw^^.CreateSurface^( pDDraw, ddsd, pSurf, nil );
    if FAILED(hr) then
    begin
        // Failure. This means DDraw isn't properly installed.
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        pdwDXVersion := 0;
        OutputDebugString( 'Couldn''t CreateSurface' + #13 );
        exit;
    end;

    // Query for the IDirectDrawSurface3 interface
    if FAILED( pSurf^^.QueryInterface^(pSurf, IID_IDirectDrawSurface3, pSurf3 ) ) then
    begin
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        exit;
    end;

    // QI for IDirectDrawSurface3 succeeded. We must be at least DX5
    pdwDXVersion := $500;


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 6.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // The IDirectDrawSurface4 interface was introduced with DX 6.0
    if FAILED( pSurf^^.QueryInterface^(pSurf, IID_IDirectDrawSurface4, pSurf4 ) ) then
    begin
        pDDraw^^.Release^(pDDraw);
        FreeLibrary( DDHinst );
        exit;
    end;

    // IDirectDrawSurface4 was create successfully. We must be at least DX6
    pdwDXVersion := $600;
    pSurf^^.Release^(pSurf);
    pDDraw^^.Release^(pDDraw);


    ///////////////////////////////////////////////////////////////////////////
    // DirectX 7.0 Checks
    ///////////////////////////////////////////////////////////////////////////

    // Check for DirectX 7 by creating a DDraw7 object
    PDirectDrawCreateEx := GetProcAddress( DDHinst, 'DirectDrawCreateEx' );
    if PDirectDrawCreateEx = nil then
    begin
        FreeLibrary( DDHinst );
        exit;
    end;

    if FAILED( PDirectDrawCreateEx^( nil, pDD7, IID_IDirectDraw7,  nil ) ) then
    begin
        FreeLibrary( DDHinst );
        exit;
    end;

    // DDraw7 was created successfully. We must be at least DX7.0
    pdwDXVersion := $700;
    pDD7^^.Release^(pDD7);


    ///////////////////////////////////////////////////////////////////////////
    // End of checks
    ///////////////////////////////////////////////////////////////////////////

    // Close open libraries and return
    FreeLibrary( DDHinst );

end;

end.